#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##Testprogramm fuer MoveHub unter pylegobst Bibliothek
##Schrittweise Test der BLE Communikation (BlueGiga oder BlueZ)
##Achtung: Bei der Ausführung des Skripts in Bash Konsole, wird mit Strg + C
##der Device Manager Thread anstelle des Main Threads beendet, was zu einer
##Exception führt.
##Achtung, Achtung: Führt man das Skript zuerst in Idle aus, lässt dann die
##Python Shell in Idle offen, dann werden keine Notifications angezeigt, wenn
##man anschließend das selbst Skript in der Bash Shell mit python3 testComm.py
##ausführt.
##
##S. Mack, 21.3.2020


from pylegoboost import bleclient as bc
from pylegoboost.utilities import str2hex
from pylegoboost import constants as C
from time import sleep
import logging

# Nachfolgende Zeile für Debugmeldungen ausschalten (level=10 bedeutet alle Meldungen)
#logging.basicConfig(level=10)
#logging.basicConfig(filename='logDatei.log', level=10)

HUB_MAC = '00:16:53:AB:64:29' # MAC-Adresse des MoveHubs

# Callbackfunktion, die nur die Notifications des MoveHubs ausgibt
def callback(handle,data):
   print("Notification on {}: {}".format(handle, str2hex(data)))

# BLE-Adapter verbinden (egal ob BlueGiga oder BlueZ)
my_client = bc.BLEClient(backend='Auto',controller='hci0',hub_mac=HUB_MAC)
sleep(0.5)

# Die Notifications vom MoveHub auf die Callbackfunktion leiten
my_client.ble_iface.set_notific_handler(C.MOVE_HUB_HW_UUID_CHAR, callback)
sleep(0.5)

# Beim MoveHub alle Notifications abonnieren bzw. aktivieren (0100 an Handle 0x0f)
my_client.ble_iface.enable_notifications()
sleep(0.5)

# BLE Device ID auslesen (als Test)
#print("Device ID: {}".format(my_client.ble_iface.read(0x07)))

print('Abbruch mit Strg + c')

try:
   while True:
      if not my_client.online():
         logging.info('MovHeub nicht mehr online. Programm wird abgebrochen.')
         break
      sleep(1)
      print('-', end='',flush=True)
except KeyboardInterrupt: # Wenn Strg + c eingegeben wird
   print()
   print('Strg + C erkannt...')
   sleep(2)
finally:
   print('...BLE-Verbindung wird beendet.')
   my_client.disconnect() # BLE-Verbindung beenden
   sleep(2)
   exit(0)

