#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Programm mit Basisbefehlen zum Testen der BLE-Kommunikation
mit einem Lego MoveHub (Lego Boost) über die Python-Bibliothek
pygatt zusammen mit einem BlueGiga BLED112 Bluetoothadapter.
**** Wichtig ist hier für Ubuntu 18.04, dass beim Starten des BLE
Dongles kein Reset durchgeführt wird - Sonst Laufzeitfehler. ****

pypi.org/project/pygatt/
github.com/peplin/pygatt/blob/master/pygatt/backends/bgapi

S. Mack, 21.3.20
"""

import pygatt
import time
from binascii import hexlify

MOVE_HUB_HARDWARE_HANDLE = 0x0E
MOVE_HUB_HARDWARE_UUID = '00001624-1212-efde-1623-785feabcd123'

def handle_data(handle, value):
    """
    handle -- integer, characteristic read handle the data was received on
    value -- bytearray, the data returned in the notification
    """
    print("Received data: %s" % hexlify(value))

adapter = pygatt.BGAPIBackend()
# BlueGiga Dongle intialisieren (inkl. Reset)
adapter.start(reset=False) # Mit reset=True Fehler
# BLE-Verbindung mit MoveHub aufbauen (MAC: 00:16:53:AB:64:29)
device = adapter.connect('00:16:53:AB:64:29')

# Notifications vom MoveHub abonnieren und an Funktion handle_data weiterleiten
# Muss in einem getrennten Prozess erfolgen bzw. wieder mit strg + c abgebrochen werden
device.subscribe("00001624-1212-ef-de-1623-785feabcd123",callback=handle_data)


# Abfrage der Characteristis
charac = device.discover_characteristics()
print("Characteristics des MoveHub: ")
for item in charac:
    print(item)
time.sleep(1)
# Abfrage Device ID ueber den Handle 0x07
print("Device ID MoveHub aus handle 0x07: ")
print(device.char_read_handle(0x07)) 
time.sleep(1)
# Abfrage Device ID über die UUID
print("Device ID MoveHub aus UUID: ")
print(device.char_read("00002a00-0000-1000-8000-00805f9b34fb"))
time.sleep(1)
#print(device.char_read("00001624-1212-ef-de-1623-785feabcd123"))
#time.sleep(1)
# LED des MoveHub auf Gruen setzen
print("LED auf Grün setzen...")
device.char_write_handle(0x0e, b'\x08\x00\x81\x32\x11\x51\x00\x05', wait_for_response=False)
time.sleep(1)
# LED des MoveHub auf Gleb setzen
print("LED auf Gelb setzen...")
device.char_write_handle(0x0e, b'\x08\x00\x81\x32\x11\x51\x00\x08', wait_for_response=False)
time.sleep(1)
# Motor a 2560 ms laufen lassen
print("Test Motor A...")
device.char_write_handle(0x0e, b'\x0c\x00\x81\x37\x11\x09\x00\x0a\x64\x64\x7f\x03', wait_for_response=False)
time.sleep(3)

print("Notifications Initialisierung: ")
# Initialisierungsnotifications ausgeben
device.char_write_handle(0x0f, b'\x01\x00', wait_for_response=True)
time.sleep(1)
# Notifications gruenr Taster aktivieren (neuer Thread oder mit strg + c beenden)
# b'060001020601': Taster gedrueckt, b'060001020600: Taste losgelassen
print("Notifications grüner Taster (während 5 s): ")
device.char_write_handle(0x0e, b'\x05\x00\x01\x02\x02', wait_for_response=True)
time.sleep(5)

print("BLE-Kommunikation beenden...")
adapter.stop() # fuehrt ein device.disconnect() aus um BLE-Verbindung mit MoveHub trennen